package industries.doublehelix.idleclock.listeners;

import java.awt.MouseInfo;
import java.awt.Point;
import java.time.Instant;

/**
 * reports last change date of mouse
 * note: for performance reasons, this won't actually listen to mouse movement but check mouse position every so often
 * and update the time if it's different than before. User won't manage to move the mouse to the exact same spot 
 * again anyway
 * @author klaue
 *
 */
public class MouseChangeListener implements Runnable {
	private long interval = 1000;
	private Point lastPosition = null;
	private Instant lastInstant = Instant.now();
	
	private volatile boolean stop = false; 
	
	/**
	 * 
	 * @param interval interval in milliseconds
	 */
	public MouseChangeListener(long interval) {
		this.interval = interval;
	}
	
	public void start() {
		new Thread(this, "MouseChangeListener").start();
	}
	
	public void stop() {
		this.stop = true;
	}
	
	@Override
	public void run() {
		// init
		this.lastPosition = MouseInfo.getPointerInfo().getLocation();
		this.lastInstant = Instant.now();
		
		while(true) {
			if (this.stop) break;
			Point currentPosition = MouseInfo.getPointerInfo().getLocation();
			// if either current or last is null 
			boolean changed = false;
			if (currentPosition == null || this.lastPosition == null) {
				changed = currentPosition != this.lastPosition;
			} else {
				changed = !currentPosition.equals(this.lastPosition);
			}
			if (changed) {
				this.lastPosition = currentPosition;
				this.lastInstant = Instant.now();
			}
			
			try {
				Thread.sleep(this.interval);
			} catch (InterruptedException e) {
				e.printStackTrace();
				break;
			}
		}
	}

	public long getInterval() {
		return this.interval;
	}

	public Point getLastPosition() {
		return this.lastPosition;
	}

	public Instant getLastInstant() {
		return this.lastInstant;
	}
}
